@@ -10,7 +10,7 @@
 #define XBLAS_CONFIG_HPP
 
 #define XTENSOR_BLAS_VERSION_MAJOR 0
-#define XTENSOR_BLAS_VERSION_MINOR 8
+#define XTENSOR_BLAS_VERSION_MINOR 10
 #define XTENSOR_BLAS_VERSION_PATCH 0
 
 #ifndef USE_CXXLAPACK
@@ -22,19 +22,12 @@
 #endif
 
 #ifdef __CLING__
-
-#pragma cling add_library_path("C:/bld/xtensor-blas_1519406255782/_b_env/Library/lib64")
-#pragma cling add_library_path("C:/bld/xtensor-blas_1519406255782/_b_env/Library/lib32")
-#pragma cling add_library_path("C:/bld/xtensor-blas_1519406255782/_b_env/Library/lib")
-
-#pragma cling load("libblas")
-#pragma cling load("liblapack")
-
+#include "xtensor-blas/xblas_config_cling.hpp"
 #endif
 
 namespace xt
 {
-    using XBLAS_INDEX = int;
+    using blas_index_t = BLAS_IDX;
 }
 
 #endif
